Class: Zlib::ZStream

Inherits:
Object
  • Object
show all
Defined in:
ext/zlib/zlib.c,
ext/zlib/zlib.c

Overview

Zlib::ZStream is the abstract class for the stream which handles the compressed data. The operations are defined in the subclasses: Zlib::Deflate for compression, and Zlib::Inflate for decompression.

An instance of Zlib::ZStream has one stream (struct zstream in the source) and two variable-length buffers which associated to the input (next_in) of the stream and the output (next_out) of the stream. In this document, “input buffer” means the buffer for input, and “output buffer” means the buffer for output.

Data input into an instance of Zlib::ZStream are temporally stored into the end of input buffer, and then data in input buffer are processed from the beginning of the buffer until no more output from the stream is produced (i.e. until avail_out > 0 after processing). During processing, output buffer is allocated and expanded automatically to hold all output data.

Some particular instance methods consume the data in output buffer and return them as a String.

Here is an ascii art for describing above:

+================ an instance of Zlib::ZStream ================+
||                                                            ||
||     +--------+          +-------+          +--------+      ||
||  +--| output |<---------|zstream|<---------| input  |<--+  ||
||  |  | buffer |  next_out+-------+next_in   | buffer |   |  ||
||  |  +--------+                             +--------+   |  ||
||  |                                                      |  ||
+===|======================================================|===+
    |                                                      |
    v                                                      |
"output data"                                         "input data"

If an error occurs during processing input buffer, an exception which is a subclass of Zlib::Error is raised. At that time, both input and output buffer keep their conditions at the time when the error occurs.

Method Catalogue

Many of the methods in this class are fairly low-level and unlikely to be of interest to users. In fact, users are unlikely to use this class directly; rather they will be interested in Zlib::Inflate and Zlib::Deflate.

The higher level methods are listed below.

  • #total_in

  • #total_out

  • #data_type

  • #adler

  • #reset

  • #finish

  • #finished?

  • #close

  • #closed?

Direct Known Subclasses

Deflate, Inflate

Instance Method Summary collapse

Instance Method Details

#adlerObject

Returns the adler-32 checksum.



1558
1559
1560
1561
1562
# File 'ext/zlib/zlib.c', line 1558

static VALUE
rb_zstream_adler(VALUE obj)
{
    return rb_uint2inum(get_zstream(obj)->stream.adler);
}

#avail_inObject

Returns bytes of data in the input buffer. Normally, returns 0.



1518
1519
1520
1521
1522
1523
1524
# File 'ext/zlib/zlib.c', line 1518

static VALUE
rb_zstream_avail_in(VALUE obj)
{
    struct zstream *z;
    TypedData_Get_Struct(obj, struct zstream, &zstream_data_type, z);
    return INT2FIX(NIL_P(z->input) ? 0 : (int)(RSTRING_LEN(z->input)));
}

#avail_outObject

Returns number of bytes of free spaces in output buffer. Because the free space is allocated automatically, this method returns 0 normally.



1492
1493
1494
1495
1496
1497
1498
# File 'ext/zlib/zlib.c', line 1492

static VALUE
rb_zstream_avail_out(VALUE obj)
{
    struct zstream *z;
    TypedData_Get_Struct(obj, struct zstream, &zstream_data_type, z);
    return rb_uint2inum(z->stream.avail_out);
}

#avail_out=(size) ⇒ Object

Allocates size bytes of free space in the output buffer. If there are more than size bytes already in the buffer, the buffer is truncated. Because free space is allocated automatically, you usually don’t need to use this method.



1506
1507
1508
1509
1510
1511
1512
1513
# File 'ext/zlib/zlib.c', line 1506

static VALUE
rb_zstream_set_avail_out(VALUE obj, VALUE size)
{
    struct zstream *z = get_zstream(obj);

    zstream_expand_buffer_into(z, FIX2INT(size));
    return size;
}

#closeObject

Closes the stream. All operations on the closed stream will raise an exception.



1416
1417
1418
1419
1420
1421
# File 'ext/zlib/zlib.c', line 1416

static VALUE
rb_zstream_end(VALUE obj)
{
    zstream_end(get_zstream(obj));
    return Qnil;
}

#closed?Boolean

Returns true if the stream is closed.

Returns:

  • (Boolean)


1576
1577
1578
1579
1580
1581
1582
# File 'ext/zlib/zlib.c', line 1576

static VALUE
rb_zstream_closed_p(VALUE obj)
{
    struct zstream *z;
    TypedData_Get_Struct(obj, struct zstream, &zstream_data_type, z);
    return ZSTREAM_IS_READY(z) ? Qfalse : Qtrue;
}

#data_typeObject

Guesses the type of the data which have been inputted into the stream. The returned value is either BINARY, ASCII, or UNKNOWN.



1549
1550
1551
1552
1553
# File 'ext/zlib/zlib.c', line 1549

static VALUE
rb_zstream_data_type(VALUE obj)
{
    return INT2FIX(get_zstream(obj)->stream.data_type);
}

#endObject

Closes the stream. All operations on the closed stream will raise an exception.



1416
1417
1418
1419
1420
1421
# File 'ext/zlib/zlib.c', line 1416

static VALUE
rb_zstream_end(VALUE obj)
{
    zstream_end(get_zstream(obj));
    return Qnil;
}

#ended?Boolean

Returns true if the stream is closed.

Returns:

  • (Boolean)


1576
1577
1578
1579
1580
1581
1582
# File 'ext/zlib/zlib.c', line 1576

static VALUE
rb_zstream_closed_p(VALUE obj)
{
    struct zstream *z;
    TypedData_Get_Struct(obj, struct zstream, &zstream_data_type, z);
    return ZSTREAM_IS_READY(z) ? Qfalse : Qtrue;
}

#finishString #finish {|chunk| ... } ⇒ nil

Finishes the stream and flushes output buffer. If a block is given each chunk is yielded to the block until the input buffer has been flushed to the output buffer.

Overloads:

  • #finishString

    Returns:

    • (String)
  • #finish {|chunk| ... } ⇒ nil

    Yields:

    • (chunk)

    Returns:

    • (nil)


1443
1444
1445
1446
1447
1448
1449
1450
1451
# File 'ext/zlib/zlib.c', line 1443

static VALUE
rb_zstream_finish(VALUE obj)
{
    struct zstream *z = get_zstream(obj);

    zstream_run(z, (Bytef*)"", 0, Z_FINISH);

    return zstream_detach_buffer(z);
}

#finished?Boolean

Returns true if the stream is finished.

Returns:

  • (Boolean)


1567
1568
1569
1570
1571
# File 'ext/zlib/zlib.c', line 1567

static VALUE
rb_zstream_finished_p(VALUE obj)
{
    return ZSTREAM_IS_FINISHED(get_zstream(obj)) ? Qtrue : Qfalse;
}

#flush_next_inObject



1458
1459
1460
1461
1462
1463
1464
1465
1466
1467
# File 'ext/zlib/zlib.c', line 1458

static VALUE
rb_zstream_flush_next_in(VALUE obj)
{
    struct zstream *z;
    VALUE dst;

    TypedData_Get_Struct(obj, struct zstream, &zstream_data_type, z);
    dst = zstream_detach_input(z);
    return dst;
}

#flush_next_outString #flush_next_out {|chunk| ... } ⇒ nil

Flushes output buffer and returns all data in that buffer. If a block is given each chunk is yielded to the block until the current output buffer has been flushed.

Overloads:

  • #flush_next_outString

    Returns:

    • (String)
  • #flush_next_out {|chunk| ... } ⇒ nil

    Yields:

    • (chunk)

    Returns:

    • (nil)


1478
1479
1480
1481
1482
1483
1484
1485
1486
# File 'ext/zlib/zlib.c', line 1478

static VALUE
rb_zstream_flush_next_out(VALUE obj)
{
    struct zstream *z;

    TypedData_Get_Struct(obj, struct zstream, &zstream_data_type, z);

    return zstream_detach_buffer(z);
}

#resetObject

Resets and initializes the stream. All data in both input and output buffer are discarded.



1427
1428
1429
1430
1431
1432
# File 'ext/zlib/zlib.c', line 1427

static VALUE
rb_zstream_reset(VALUE obj)
{
    zstream_reset(get_zstream(obj));
    return Qnil;
}

#stream_end?Boolean

Returns true if the stream is finished.

Returns:

  • (Boolean)


1567
1568
1569
1570
1571
# File 'ext/zlib/zlib.c', line 1567

static VALUE
rb_zstream_finished_p(VALUE obj)
{
    return ZSTREAM_IS_FINISHED(get_zstream(obj)) ? Qtrue : Qfalse;
}

#total_inObject

Returns the total bytes of the input data to the stream. FIXME



1529
1530
1531
1532
1533
# File 'ext/zlib/zlib.c', line 1529

static VALUE
rb_zstream_total_in(VALUE obj)
{
    return rb_uint2inum(get_zstream(obj)->stream.total_in);
}

#total_outObject

Returns the total bytes of the output data from the stream. FIXME



1538
1539
1540
1541
1542
# File 'ext/zlib/zlib.c', line 1538

static VALUE
rb_zstream_total_out(VALUE obj)
{
    return rb_uint2inum(get_zstream(obj)->stream.total_out);
}