Exception: Zlib::Error
- Inherits:
-
StandardError
- Object
- StandardError
- Zlib::Error
- Defined in:
- ext/zlib/zlib.c,
ext/zlib/zlib.c
Overview
The superclass for all exceptions raised by Ruby/zlib.
The following exceptions are defined as subclasses of Zlib::Error. These exceptions are raised when zlib library functions return with an error status.
- Zlib::StreamEnd
- Zlib::NeedDict
- Zlib::DataError
- Zlib::StreamError
- Zlib::MemError
- Zlib::BufError
- Zlib::VersionError
- Zlib::InProgressError
Direct Known Subclasses
BufError, DataError, GzipFile::Error, InProgressError, MemError, NeedDict, StreamEnd, StreamError, VersionError